import argparse
import asyncio
import sys
import py_auth_micro

from typing import Optional

from ._version import __version__
from ._load_config import load_config
from ._runner import run


def cli() -> Optional[str]:
    parser = argparse.ArgumentParser(
        prog="py_auth_amqp_wrapper",
        description="This will start an auth server based on py_auth_micro and connect it to an AMQP message Broker",
    )
    parser.add_argument(
        "--config", "-c", help="path to the configuration file", default="./config.json", type=str)
    parser.add_argument(
        "--version", "-v", help="displays the package version and quits", action="store_true")
    parser.add_argument("--disable_existing_loggers", "-d",
                        help="disables existing loggers", action="store_true")

    inputvars = parser.parse_args()

    if inputvars.version:
        print(f"py_auth_micro=={__version__}")
        return None

    return inputvars.config, inputvars.disable_existing_loggers


if __name__ == "__main__":
    config, disable_existing_loggers = cli()

    if config is None:
        sys.exit(0)

        # configure DB
    print(f"""                             _   _     
                            | | | |    
  _ __  _   _     __ _ _   _| |_| |__  
 | '_ \| | | |   / _` | | | | __| '_ \ 
 | |_) | |_| |  | (_| | |_| | |_| | | |
 | .__/ \__, |   \__,_|\__,_|\__|_| |_|
 | |     __/ |_____                    
 |_|    |___/______|
       py_auth_micro: {py_auth_micro.__version__}
py_auth_amqp_wrapper: {__version__}
 
Starting Up!""")

    print(f"loading config file: '{config}'")
    try:
        config = load_config(config)
    except Exception as exc:
        print(exc)
        print("could not load config")
        sys.exit(78)
    print("loaded config")
    print("\n\nstarting up")
    asyncio.run(run(**config, disable_existing_loggers=disable_existing_loggers))



import logging
import traceback

from tortoise import Tortoise

from jwt_helper import JWTEncoder, JWTValidator
from amqp_helper import AMQPConfig, AMQPService, new_amqp_func
from py_auth_micro.Config import DBConfig, AppConfig, LDAPConfig
from py_auth_micro.WorkFlows import SessionWorkflow, UserWorkflow, GroupWorkflow

from ._getlogger import getlogger


async def run(
    *,
    queue_settings: dict,
    jwt_encoder: JWTEncoder = None,
    jwt_validator: JWTValidator = None,
    db_config: DBConfig = DBConfig,
    app_config: AppConfig = AppConfig,
    ldap_config: LDAPConfig = None,
    amqp_config: AMQPConfig = AMQPConfig,
    log_config: logging.Logger = None,
    disable_existing_loggers=False,
):
    """This Function will run the auth_service

    Args:
        queue_settings (dict): Settings for the QUEUE-Names to use
        jwt_encoder (JWTEncoder, optional): JWT Encoder used for creating JWTs. Defaults to None.
        jwt_validator (JWTValidator, optional): JWTValidator used to verify JWTS. Defaults to None.
        db_config (DBConfig, optional): Database Configuration. Defaults to DBConfig.
        app_config (AppConfig, optional): General App Settings. Defaults to AppConfig.
        ldap_config (LDAPConfig, optional): Configuration for LDAP Interactions. Defaults to None.
        amqp_config (AMQPConfig, optional): Configuration for the AMQP Protocoll. Defaults to AMQPConfig.
        log_config (logging.Logger, optional): If given will configure the root logger. Defaults to None.
        disable_existing_loggers (bool, optional): Disables existing loggers. Defaults to False.
    """
    await Tortoise.init(
        config={
            "connections": db_config.to_dict("default"),
            "apps": {
                "models": {
                    "models": ["py_auth_micro.Models"],
                    "default_connection": "default",
                }
            },
        }
    )

    logger = logging.getLogger("py_auth_amqp_wrapper")
    if log_config is not None:
        logger = getlogger(
            **log_config, disable_existing_loggers=disable_existing_loggers
        )
        logger.info("setup logger")

    sessionwf = SessionWorkflow(ldap_config, jwt_encoder, jwt_validator, app_config)
    session_workflow_get_access_token = new_amqp_func(
        queue_settings["session_workflow_get_access_token"], sessionwf.get_access_token
    )
    session_workflow_login = new_amqp_func(
        queue_settings["session_workflow_login"], sessionwf.login
    )
    session_workflow_logout = new_amqp_func(
        queue_settings["session_workflow_logout"], sessionwf.logout
    )

    userwf = UserWorkflow(jwt_validator, app_config)
    user_workflow_admin_create_user = new_amqp_func(
        queue_settings["user_workflow_admin_create_user"], userwf.admin_create_user
    )
    user_workflow_register_user = new_amqp_func(
        queue_settings["user_workflow_register_user"], userwf.register_user
    )
    user_workflow_delete_user = new_amqp_func(
        queue_settings["user_workflow_delete_user"], userwf.delete_user
    )
    user_workflow_change_user = new_amqp_func(
        queue_settings["user_workflow_change_user"], userwf.change_user
    )
    user_workflow_get_all = new_amqp_func(
        queue_settings["user_workflow_get_all"], userwf.get_all
    )
    user_workflow_get_user = new_amqp_func(
        queue_settings["user_workflow_get_user"], userwf.get_user
    )

    groupwf = GroupWorkflow(jwt_validator, app_config)
    group_workflow_add_user_to_group = new_amqp_func(
        queue_settings["group_workflow_add_user_to_group"], groupwf.add_user_to_group
    )
    group_workflow_remove_user_from_group = new_amqp_func(
        queue_settings["group_workflow_remove_user_from_group"],
        groupwf.remove_user_from_group,
    )
    group_workflow_create_group = new_amqp_func(
        queue_settings["group_workflow_create_group"], groupwf.create_group
    )
    group_workflow_delete_group = new_amqp_func(
        queue_settings["group_workflow_delete_group"], groupwf.delete_group
    )

    @session_workflow_logout.exception_handler(TypeError)
    @session_workflow_login.exception_handler(TypeError)
    @session_workflow_get_access_token.exception_handler(TypeError)
    async def handle_exception(*args, exc: Exception, **kwargs):
        logger.exception(exc, kwargs)
        return {"resp_code": 400, "resp_data": {"msg": "Invalid Data"}}

    @session_workflow_login.exception_handler(ValueError, PermissionError)
    async def handle_login_fail(*args, exc: Exception, **kwargs):
        logger.exception(exc, kwargs)
        return {"resp_code": 401, "resp_data": {"msg": "Could not login"}}

    @user_workflow_admin_create_user.exception_handler(Exception)
    @user_workflow_register_user.exception_handler(Exception)
    @user_workflow_delete_user.exception_handler(Exception)
    @user_workflow_change_user.exception_handler(Exception)
    @user_workflow_get_all.exception_handler(Exception)
    @user_workflow_get_user.exception_handler(Exception)
    @group_workflow_add_user_to_group.exception_handler(Exception)
    @group_workflow_remove_user_from_group.exception_handler(Exception)
    @group_workflow_create_group.exception_handler(Exception)
    @group_workflow_delete_group.exception_handler(Exception)
    @session_workflow_logout.exception_handler(Exception)
    @session_workflow_login.exception_handler(Exception)
    @session_workflow_get_access_token.exception_handler(Exception)
    async def handle_exception(*args, exc: Exception, **kwargs):
        logger.exception(exc, kwargs)
        return {"resp_code": 500, "resp_data": {"msg": f"something went wrong! {exc}"}}

    service = await AMQPService().connect(amqp_config)

    await service.register_function(session_workflow_get_access_token)
    await service.register_function(session_workflow_login)
    await service.register_function(session_workflow_logout)
    await service.register_function(user_workflow_admin_create_user)
    await service.register_function(user_workflow_register_user)
    await service.register_function(user_workflow_delete_user)
    await service.register_function(user_workflow_change_user)
    await service.register_function(user_workflow_get_all)
    await service.register_function(user_workflow_get_user)
    await service.register_function(group_workflow_add_user_to_group)
    await service.register_function(group_workflow_remove_user_from_group)
    await service.register_function(group_workflow_create_group)
    await service.register_function(group_workflow_delete_group)

    await service.serve()

    return



import json
import base64
import logging

from jwt_helper import Issuer, load_key_from_disk, JWTEncoder, JWTValidator, SignMethod
from py_auth_micro.Config import DBConfig, AppConfig, LDAPConfig
from amqp_helper import AMQPConfig
from pathlib import Path

from ._getlogger import getlogger

SECRET_METHODS = {
    "plain": lambda val: val,
    "base64plain": lambda val: base64.b64decode(val.encode("utf-8")).decode("utf8"),
    "base64key": base64.b64decode,
    "keyfile": load_key_from_disk,
}


def load_config(path_to_file: str) -> dict:
    """This Functions loads our Json Config and converts it into an dict.

    Some Configuration Parts will be converted into the corresponding config clases like LDAPConfig, APPConfig, DBConfig and AMQPConfig.

    Args:
        path_to_file (str): Filepath of the configuration File

    Raises:
        ValueError: If a Config Part is missing or invalid

    Returns:
        dict: Dictionary Containing all relevant Configuration Parts
    """
    raw_config: dict = json.loads(Path(path_to_file).read_text())

    jwt_validation = raw_config.get("jwt_validation", None)
    jwt_creation = raw_config.get("jwt_creation", None)
    ldap_settings = raw_config.get("ldap_settings", None)
    db_settings = raw_config.get("db_settings", None)
    amqp_settings = raw_config.get("amqp_settings", None)
    app_config = raw_config.get("app_config", None)
    queue_settings = raw_config.get("queue_settings", None)
    log_settings = raw_config.get("log_settings", None)

    config_dict = {}

    default_queue_names = {
        "session_workflow_login": "login",
        "session_workflow_logout": "logout",
        "session_workflow_get_access_token": "token",
        "user_workflow_register_user": "user_register",
        "user_workflow_admin_create_user": "user_admin_create",
        "user_workflow_delete_user": "user_delete",
        "user_workflow_change_user": "user_change",
        "user_workflow_get_all": "user_all",
        "user_workflow_get_user": "user_get",
        "group_workflow_add_user_to_group": "group_add_user",
        "group_workflow_remove_user_from_group": "group_remove_user",
        "group_workflow_create_group": "group_create",
        "group_workflow_delete_group": "group_delete",
    }

    if jwt_validation is not None:
        jwt_validator = JWTValidator()

        for issuer in jwt_validation:
            # translate sign methods
            methods = []
            for method in issuer["methods"]:
                methods.append(SignMethod[method])
            # load key the specified way
            key_type = issuer["secret"]["type"]
            secret_value = issuer["secret"]["value"]

            if not (
                key_type == "keyfile"
                or key_type == "base64plain"
                or key_type == "base64key"
                or key_type == "plain"
            ):
                raise ValueError(f"keytype {key_type} not supported")
            secret = SECRET_METHODS[key_type](secret_value)

            jwt_validator.add_issuer(Issuer(issuer["name"], secret, methods))

        config_dict["jwt_validator"] = jwt_validator

    if jwt_creation is not None:

        # get secret for signing
        key_type = jwt_creation["secret"]["type"]
        secret_value = jwt_creation["secret"]["value"]
        key_secret = jwt_creation["secret"].get("secret", None)

        if not (
            key_type == "keyfile"
            or key_type == "base64plain"
            or key_type == "base64key"
            or key_type == "plain"
        ):
            raise ValueError(f"keytype {key_type} not supported")

        if key_secret is not None:
            secret = SECRET_METHODS[key_type](secret_value, key_secret)
        else:
            secret = SECRET_METHODS[key_type](secret_value)

        # get sign method for signing
        sign_method = SignMethod[jwt_creation["signmethod"]]

        # create jwtencoder
        config_dict["jwt_encoder"] = JWTEncoder(
            jwt_creation["issuer"], sign_method, secret
        )

    if ldap_settings is not None:
        config_dict["ldap_config"] = LDAPConfig(**ldap_settings)

    if db_settings is not None:
        config_dict["db_config"] = DBConfig(**db_settings)

    if app_config is not None:
        config_dict["app_config"] = AppConfig(**app_config)

    if amqp_settings is not None:
        config_dict["amqp_config"] = AMQPConfig(**amqp_settings)

    if queue_settings is not None:
        for key, value in queue_settings.items():
            default_queue_names[key] = value

    if log_settings is not None:

        LEVELMAP = {
            "NOTSET": logging.NOTSET,
            "DEBUG": logging.DEBUG,
            "INFO": logging.INFO,
            "WARNING": logging.WARNING,
            "ERROR": logging.ERROR,
            "CRITICAL": logging.CRITICAL,
        }

        if log_settings.get("disable_aio_pika", False):
            pass

        if log_settings.get("disable_tortoise_orm", False):
            logging.getLogger("db_client").disabled = True
            logging.getLogger("tortoise").disabled = True

        level = LEVELMAP[log_settings.get("level", "INFO")]
        handler = log_settings.get("handler", None)
        handler_settings = log_settings.get("handler_settings", {})

        if handler_settings.get("amqp", None) is not None:
            amqp_settings = AMQPConfig(**handler_settings["amqp"])
            handler_settings = {"amqp_config": amqp_settings}

        config_dict["log_config"] = {
            "log_level": level, "handler": handler, "handler_config": handler_settings}

    config_dict["queue_settings"] = default_queue_names

    return config_dict



import logging
from logging.config import dictConfig
import importlib


def getlogger(
    log_level: int,
    handler: str = None,
    handler_config: dict = {},
    disable_existing_loggers: bool = False,
) -> logging.Logger:

    logger = dictConfig(
        {"version": 1, "disable_existing_loggers": disable_existing_loggers}
    )

    logger = logging.getLogger()

    logger.setLevel(log_level)
    if handler is not None:
        handler_str = handler
        handler_pkg = ".".join(handler_str.split(".")[:-1])
        handler_name = handler_str.split(".")[-1]
        handler_class = getattr(importlib.import_module(handler_pkg), handler_name)
        handler_instance = handler_class(**handler_config)
        logger.addHandler(handler_instance)
    return logger
